@@ -20,11 +20,13 @@
 #include <dlfcn.h>
 #endif
 
+/*! \brief Shared state received from the driver */
+plugin_shared_state* shared_state_from_driver = NULL;
 
 /*! \brief Enable support for the PLUG method */
-int enable_plug_support() {
-  new_method(MDI_LINK);
-  method* this_method = get_method(MDI_LINK);
+int enable_plug_support( int code_id ) {
+  new_method(code_id, MDI_LINK);
+  method* this_method = get_method(code_id, MDI_LINK);
   this_method->on_selection = plug_on_selection;
   this_method->on_accept_communicator = plug_on_accept_communicator;
   this_method->on_send_command = plug_on_send_command;
@@ -76,8 +78,8 @@ int plug_on_send_command(const char* command, MDI_Comm comm, int* skip_flag) {
   code* this_code = get_code(current_code);
   communicator* this_comm = get_communicator(current_code, comm);
   library_data* libd = (library_data*) this_comm->method_data;
-  int iengine = libd->connected_code;
-  code* engine_code = get_code(iengine);
+  //int iengine = libd->connected_code;
+  //code* engine_code = get_code(iengine);
   int ret = 0;
 
   // Check whether MPI has been initialized
@@ -89,7 +91,7 @@ int plug_on_send_command(const char* command, MDI_Comm comm, int* skip_flag) {
 
   // broadcast the command to each rank
   char* command_bcast = malloc( MDI_COMMAND_LENGTH * sizeof(char) );
-  if ( engine_code->intra_rank == 0 ) {
+  //if ( engine_code->intra_rank == 0 ) {
     int ichar;
     for ( ichar=0; ichar < MDI_COMMAND_LENGTH; ichar++) {
       command_bcast[ichar] = '\0';
@@ -97,25 +99,35 @@ int plug_on_send_command(const char* command, MDI_Comm comm, int* skip_flag) {
     for ( ichar=0; ichar < strlen(command) && ichar < MDI_COMMAND_LENGTH; ichar++ ) {
       command_bcast[ichar] = command[ichar];
     }
-  }
+  //}
+  /*
   if ( mpi_init_flag == 1) {
-    MPI_Bcast( &command_bcast[0], MDI_COMMAND_LENGTH, MPI_CHAR, 0, engine_code->intra_MPI_comm);
+    //MPI_Bcast( &command_bcast[0], MDI_COMMAND_LENGTH, MPI_CHAR, 0, engine_code->intra_MPI_comm);
+    MPI_Bcast( &command_bcast[0], MDI_COMMAND_LENGTH, MPI_CHAR, 0, *(MPI_Comm*)libd->shared_state->mpi_comm_ptr);
   }
+  */
 
   // ensure that the driver is the current code
-  library_set_driver_current();
+  library_set_driver_current(comm);
 
   // set the command for the engine to execute
   library_set_command(command_bcast, comm);
 
   if ( command_bcast[0] == '<' ) {
     // execute the command, so that the data from the engine can be received later by the driver
-    ret = library_execute_command(comm);
+    //ret = library_execute_command(comm);
+
+    libd->shared_state->engine_activate_code( libd->shared_state->engine_code_id );
+
+    libd->shared_state->lib_execute_command(libd->shared_state->engine_mdi_comm);
     if ( ret != 0 ) {
       mdi_error("Error in MDI_Send_Command: Unable to execute receive command through library");
       free( command_bcast );
       return ret;
     }
+
+    libd->shared_state->driver_activate_code( libd->shared_state->driver_code_id );
+
     *skip_flag = 1;
   }
   else if ( command_bcast[0] == '>' ) {
@@ -130,12 +142,19 @@ int plug_on_send_command(const char* command, MDI_Comm comm, int* skip_flag) {
   }
   else {
     // this is a command that neither sends nor receives data, so execute it now
-    ret = library_execute_command(comm);
+    //ret = library_execute_command(comm);
+
+    libd->shared_state->engine_activate_code( libd->shared_state->engine_code_id );
+
+    libd->shared_state->lib_execute_command(libd->shared_state->engine_mdi_comm);
     if ( ret != 0 ) {
       mdi_error("Error in MDI_Send_Command: Unable to execute command through library");
       free( command_bcast );
       return ret;
     }
+
+    libd->shared_state->driver_activate_code( libd->shared_state->driver_code_id );
+
     *skip_flag = 1;
   }
 
@@ -156,29 +175,37 @@ int plug_after_send_command(const char* command, MDI_Comm comm) {
 int plug_on_recv_command(MDI_Comm comm) {
   int ret = 0;
   int iengine = current_code;
+  code* this_code = get_code(current_code);
   communicator* engine_comm = get_communicator(current_code, comm);
 
   // get the driver code to which this communicator connects
   library_data* libd = (library_data*) engine_comm->method_data;
   int idriver = libd->connected_code;
-  code* driver_code = get_code(idriver);
 
-  MDI_Comm driver_comm_handle = library_get_matching_handle(comm);
-  communicator* driver_comm = get_communicator(idriver, driver_comm_handle);
-  library_data* driver_lib = (library_data*) driver_comm->method_data;
+  //MDI_Comm driver_comm_handle = library_get_matching_handle(comm);
+  //communicator* driver_comm = get_communicator(idriver, driver_comm_handle);
+  //library_data* driver_lib = (library_data*) driver_comm->method_data;
+
+  // Copy the execute_command function to a shared location
+  libd->shared_state->execute_command = this_code->execute_command;
+  libd->shared_state->execute_command_obj = this_code->execute_command_obj;
 
   // set the current code to the driver
-  current_code = idriver;
+//  current_code = idriver;
+//  current_code = libd->shared_state->driver_code_id;
+  libd->shared_state->driver_activate_code( libd->shared_state->driver_code_id );
 
-  void* class_obj = driver_lib->driver_callback_obj;
-  ret = driver_lib->driver_node_callback(&driver_lib->mpi_comm, driver_comm_handle, class_obj);
+  //void* class_obj = driver_lib->driver_callback_obj;
+  ret = libd->shared_state->driver_node_callback(libd->shared_state->mpi_comm_ptr, libd->shared_state->driver_mdi_comm, libd->shared_state->driver_callback_obj);
   if ( ret != 0 ) {
     mdi_error("PLUG error in on_recv_command: driver_node_callback failed");
     return ret;
   }
 
   // set the current code to the engine
-  current_code = iengine;
+//  current_code = iengine;
+//  current_code = libd->shared_state->engine_code_id;
+  libd->shared_state->engine_activate_code( libd->shared_state->engine_code_id );
 
   return 0;
 }
@@ -214,7 +241,8 @@ int library_load_init(const char* plugin_name, void* mpi_comm_ptr,
   snprintf(plugin_path, PLUGIN_PATH_LENGTH, "%s/%s.py", this_code->plugin_path, plugin_name);
   if ( file_exists(plugin_path) ) {
     libd->is_python = 1;
-    ret = python_plugin_init( plugin_name, plugin_path, mpi_comm_ptr );
+    libd->shared_state->engine_language = MDI_LANGUAGE_PYTHON;
+    ret = python_plugin_init( plugin_name, plugin_path, mpi_comm_ptr, libd->shared_state );
     if ( ret != 0 ) {
       mdi_error("Error in python_plugin_init");
       return -1;
@@ -274,11 +302,12 @@ int library_load_init(const char* plugin_name, void* mpi_comm_ptr,
 #endif
 
     // Initialize an instance of the plugin
-    ret = libd->plugin_init();
+    ret = libd->plugin_init( libd->shared_state );
     if ( ret != 0 ) {
       mdi_error("MDI plugin init function returned non-zero exit code");
       return -1;
     }
+
   }
 
   // free memory from loading the plugin's initialization function
@@ -306,7 +335,7 @@ int library_parse_options(const char* options, library_data* libd) {
   libd->plugin_options_allocated = 1;
 
   // determine the number of arguments
-  libd->plugin_argc = 0;
+  libd->shared_state->plugin_argc = 0;
   int ichar;
   int in_argument = 0; // was the previous character part of an argument, or just whitespace?
   int in_single_quotes = 0; // was the previous character part of a single quote?
@@ -317,14 +346,14 @@ int library_parse_options(const char* options, library_data* libd) {
         mdi_error("Unterminated double quotes received in MDI_Launch_plugin \"options\" argument.");
       }
       if ( in_argument ) {
-        libd->plugin_argc++;
+        libd->shared_state->plugin_argc++;
       }
       in_argument = 0;
     }
     else if (libd->plugin_options[ichar] == ' ') {
       if ( ! in_double_quotes && ! in_single_quotes ) {
         if ( in_argument ) {
-          libd->plugin_argc++;
+          libd->shared_state->plugin_argc++;
         }
         in_argument = 0;
         libd->plugin_options[ichar] = '\0';
@@ -352,18 +381,18 @@ int library_parse_options(const char* options, library_data* libd) {
   }
 
   // construct pointers to all of the arguments
-  libd->plugin_argv = malloc( libd->plugin_argc * sizeof(char*) );
-  libd->plugin_argv_allocated = 1;
+  libd->shared_state->plugin_argv = malloc( libd->shared_state->plugin_argc * sizeof(char*) );
+  libd->shared_state->plugin_argv_allocated = 1;
   int iarg = 0;
   for (ichar=0; ichar < options_len; ichar++) {
     if ( libd->plugin_options[ichar] != '\0' ) {
       if ( ichar == 0 || libd->plugin_options[ichar-1] == '\0' ) {
-        libd->plugin_argv[iarg] = &libd->plugin_options[ichar];
+        libd->shared_state->plugin_argv[iarg] = &libd->plugin_options[ichar];
         iarg++;
       }
     }
   }
-  if ( iarg != libd->plugin_argc ) {
+  if ( iarg != libd->shared_state->plugin_argc ) {
     mdi_error("Programming error: unable to correctly parse the MDI_Launch_plugin \"options\" argument.");
   }
 
@@ -379,19 +408,23 @@ int library_launch_plugin(const char* plugin_name, const char* options, void* mp
                           MDI_Driver_node_callback_t driver_node_callback,
                           void* driver_callback_object) {
   int ret;
-  int driver_code_id = current_code;
-  code* this_code = get_code(driver_code_id);
+  code* this_code = get_code(current_code);
   MPI_Comm mpi_comm = *(MPI_Comm*) mpi_comm_ptr;
 
-
-
-
   // initialize a communicator for the driver
   int icomm = library_initialize();
   communicator* driver_comm = get_communicator(current_code, icomm);
   library_data* libd = (library_data*) driver_comm->method_data;
   libd->connected_code = (int)codes.size;
 
+  // allocate data that is shared between the driver and the plugin
+  libd->shared_state = malloc(sizeof(plugin_shared_state));
+  libd->shared_state->plugin_argc = 0;
+  libd->shared_state->plugin_argv_allocated = 0;
+  libd->shared_state->buf_allocated = 0;
+  libd->shared_state->driver_code_id = current_code;
+  libd->shared_state->engine_language = MDI_LANGUAGE_C;
+
   MDI_Comm comm;
   ret = MDI_Accept_Communicator(&comm);
   if ( ret != 0 || comm == MDI_COMM_NULL ) {
@@ -410,8 +443,15 @@ int library_launch_plugin(const char* plugin_name, const char* options, void* mp
   library_parse_options(options, libd);
 
   // Assign the global command-line options variables to the values for this plugin  
-  plugin_argc = libd->plugin_argc;
-  plugin_argv = libd->plugin_argv;
+  plugin_argc = libd->shared_state->plugin_argc;
+  plugin_argv = libd->shared_state->plugin_argv;
+
+
+  libd->shared_state->mpi_comm_ptr = &libd->mpi_comm;
+  libd->shared_state->driver_node_callback = libd->driver_node_callback;
+  libd->shared_state->driver_mdi_comm = driver_comm->id;
+  libd->shared_state->driver_activate_code = library_activate_code;
+  libd->shared_state->driver_callback_obj = libd->driver_callback_obj;
 
   //
   // Get the path to the plugin
@@ -431,26 +471,43 @@ int library_launch_plugin(const char* plugin_name, const char* options, void* mp
   /*************************************************/
   plugin_mode = 1;
 
-  library_load_init(plugin_name, mpi_comm_ptr, libd, 0);
+  ret = library_load_init(plugin_name, mpi_comm_ptr, libd, 0);
+  if ( ret != 0 ) {
+    free( plugin_path );
+    free( plugin_init_name );
+    return ret;
+  }
 
-  if (libd->is_python == 0 ) {
+  // store a couple of values from libd, before we delete it
+  int is_python = libd->is_python;
+#ifdef _WIN32
+  HINSTANCE plugin_handle = libd->plugin_handle;
+#else
+  void* plugin_handle = libd->plugin_handle;
+#endif
+
+  plugin_mode = 0;
+//  current_code = libd->shared_state->driver_code_id;
+//  current_code = libd->shared_state->driver_code_id;
+  libd->shared_state->driver_activate_code( libd->shared_state->driver_code_id );
+
+  // Delete the driver's communicator to the engine
+  // This will also delete the engine code and its communicator
+  delete_communicator(libd->shared_state->driver_code_id, comm);
+
+  if (is_python == 0 ) {
   // Close the plugin library
 #ifdef _WIN32
-    FreeLibrary( libd->plugin_handle );
+    FreeLibrary( plugin_handle );
 #else
-    dlclose( libd->plugin_handle );
+    dlclose( plugin_handle );
 #endif
   }
 
   /*************************************************/
   /**************** END PLUGIN MODE ****************/
   /*************************************************/
-  plugin_mode = 0;
-  current_code = driver_code_id;
 
-  // Delete the driver's communicator to the engine
-  // This will also delete the engine code and its communicator
-  delete_communicator(driver_code_id, comm);
 
   // free memory from loading the plugin's initialization function
   free( plugin_path );
@@ -491,8 +548,8 @@ int library_open_plugin(const char* plugin_name, const char* options, void* mpi_
   library_parse_options(options, libd);
 
   // Assign the global command-line options variables to the values for this plugin  
-  plugin_argc = libd->plugin_argc;
-  plugin_argv = libd->plugin_argv;
+  plugin_argc = libd->shared_state->plugin_argc;
+  plugin_argv = libd->shared_state->plugin_argv;
   plugin_unedited_options = libd->plugin_unedited_options;
 
   //
@@ -511,12 +568,19 @@ int library_open_plugin(const char* plugin_name, const char* options, void* mpi_
   plugin_mode = 1;
 
   library_load_init(plugin_name, mpi_comm_ptr, libd, 1);
+  if ( ret != 0 ) {
+    free( plugin_path );
+    free( plugin_init_name );
+    return ret;
+  }
 
   /*************************************************/
   /**************** END PLUGIN MODE ****************/
   /*************************************************/
   plugin_mode = 0;
-  current_code = driver_code_id;
+//  current_code = driver_code_id;
+//  current_code = libd->shared_state->driver_code_id;
+  libd->shared_state->driver_activate_code( libd->shared_state->driver_code_id );
 
   // Delete the driver's communicator to the engine
   // This will also delete the engine code and its communicator
@@ -573,34 +637,43 @@ int library_initialize() {
 
   // allocate the method data
   library_data* libd = malloc(sizeof(library_data));
-  libd->connected_code = -1;
-  libd->buf_allocated = 0;
+  //libd->connected_code = -1;
+  //libd->buf_allocated = 0;
   libd->execute_on_send = 0;
   libd->mpi_comm = MPI_COMM_NULL;
-  libd->plugin_argv_allocated = 0;
   libd->plugin_options_allocated = 0;
-  libd->plugin_argc = 0;
   libd->plugin_options = NULL;
   libd->plugin_unedited_options = NULL;
+
   new_comm->method_data = libd;
 
   // if this is an engine, go ahead and set the driver as the connected code
   if ( strcmp(this_code->role, "ENGINE") == 0 ) {
-    int engine_code = current_code;
-    library_set_driver_current();
-    int driver_code_id = current_code;
-    libd->connected_code = driver_code_id;
-    current_code = engine_code;
+    //int engine_code = current_code;
+    //library_set_driver_current();
+    //int driver_code_id = current_code;
+    //libd->connected_code = driver_code_id;
+    //current_code = engine_code;
+    libd->shared_state = shared_state_from_driver;
+    libd->shared_state->engine_mdi_comm = new_comm->id;
+    libd->shared_state->delete_engine = library_delete_engine;
+    libd->shared_state->engine_activate_code = library_activate_code;
+    libd->shared_state->lib_execute_command = library_execute_command;
+    libd->shared_state->engine_code_id = current_code;
+//    current_code = libd->shared_state->engine_code_id;
 
     // set the engine's mpi communicator
-    if ( plugin_mode ) {
+    //if ( plugin_mode ) {
       // get the driver's library data
-      code* driver_code = get_code(driver_code_id);
-      MDI_Comm matching_handle = library_get_matching_handle(comm_id);
-      communicator* driver_comm = get_communicator(driver_code->id, matching_handle);
-      library_data* driver_libd = (library_data*) driver_comm->method_data;
+      //code* driver_code = get_code(driver_code_id);
+      //MDI_Comm matching_handle = library_get_matching_handle(comm_id);
+      //communicator* driver_comm = get_communicator(driver_code->id, matching_handle);
+      //library_data* driver_libd = (library_data*) driver_comm->method_data;
 
-      libd->mpi_comm = driver_libd->mpi_comm;
+
+
+      //libd->mpi_comm = driver_libd->mpi_comm;
+      libd->mpi_comm = *(MPI_Comm*)libd->shared_state->mpi_comm_ptr;
       this_code->intra_MPI_comm = libd->mpi_comm;
 
       // check whether MPI has been initialized
@@ -617,7 +690,10 @@ int library_initialize() {
       else {
         this_code->intra_rank = 0;
       }
-    }
+      
+      libd->shared_state->intra_rank = this_code->intra_rank;
+    //}
+
   }
 
   return new_comm->id;
@@ -627,29 +703,14 @@ int library_initialize() {
 /*! \brief Set the driver as the current code
  *
  */
-int library_set_driver_current() {
+int library_set_driver_current(MDI_Comm comm) {
   code* this_code = get_code(current_code);
+  communicator* this_comm = get_communicator(current_code, comm);
+  library_data* libd = (library_data*) this_comm->method_data;
+
+  //current_code = libd->shared_state->driver_code_id;
+  libd->shared_state->driver_activate_code( libd->shared_state->driver_code_id );
 
-  // check if the current code is an ENGINE that is linked as a LIBRARY
-  if ( strcmp(this_code->role, "ENGINE") == 0 ) {
-    if ( this_code->is_library == 1 || this_code->is_library == 2 ) {
-      // the calling code must actually be the driver, so update current_code
-      int icode;
-      int found_driver = 0;
-      for ( icode = 0; icode < codes.size; icode++ ) {
-	code* other_code = vector_get(&codes, icode);
-	if ( strcmp(other_code->role, "DRIVER") == 0 ) {
-	  current_code = icode;
-	  found_driver = 1;
-	}
-      }
-      // confirm that the driver was found
-      if ( found_driver == 0 ) {
-	mdi_error("MDI_Accept_Communicator could not locate the driver; was MDI_Init called by the driver?");
-	return 1;
-      }
-    }
-  }
   return 0;
 }
 
@@ -659,11 +720,13 @@ int library_set_driver_current() {
  */
 int library_accept_communicator() {
   code* this_code = get_code(current_code);
+  /*
   if ( this_code->called_set_execute_command_func && (! plugin_mode) ) {
     // library codes are not permitted to call MDI_Accept_communicator after calling
     // MDI_Set_execute_command_func, so assume that this call is being made by the driver
     library_set_driver_current();
   }
+  */
   this_code = get_code(current_code);
 
   // if this is a DRIVER, check if there are any ENGINES that are linked to it
@@ -704,44 +767,6 @@ int library_accept_communicator() {
 }
 
 
-/*! \brief Get the handle to the matching communicator on the code to which comm points
- *
- * If running with MPI, this function must be called only by rank \p 0.
- * The function returns \p 0 on a success.
- *
- * \param [in]       comm
- *                   MDI communicator associated with the linked code.
- */
-int library_get_matching_handle(MDI_Comm comm) {
-  communicator* this = get_communicator(current_code, comm);
-
-  // get the engine code to which this communicator connects
-  library_data* libd = (library_data*) this->method_data;
-  int iengine = libd->connected_code;
-  code* engine_code = get_code(iengine);
-
-  // identify the communicator on the engine that connects to the driver
-  int icomm;
-  int found_self = 0;
-  int engine_comm_handle = 0;
-  for ( icomm = 0; icomm < engine_code->comms->size; icomm++ ) {
-    communicator* engine_comm = vector_get(engine_code->comms, icomm);
-    library_data* engine_lib = (library_data*) engine_comm->method_data;
-    if ( engine_lib->connected_code == current_code ) {
-      found_self = 1;
-      engine_comm_handle = engine_comm->id;
-    }
-  }
-
-  // ensure that the communicator was found
-  if ( found_self == 0 ) {
-    mdi_error("Could not find communicator for engine; Did the engine call MDI_Init?"); 
-    return 1;
-  }
-
-  return engine_comm_handle;
-}
-
 
 /*! \brief Set the next command that will be executed through the library communicator
  *
@@ -754,20 +779,20 @@ int library_get_matching_handle(MDI_Comm comm) {
  *                   MDI communicator associated with the intended recipient code.
  */
 int library_set_command(const char* command, MDI_Comm comm) {
-  int idriver = current_code;
   communicator* this = get_communicator(current_code, comm);
 
   // get the engine code to which this communicator connects
   library_data* libd = (library_data*) this->method_data;
-  int iengine = libd->connected_code;
+  //int iengine = libd->connected_code;
 
   // get the matching engine communicator
-  MDI_Comm engine_comm_handle = library_get_matching_handle(comm);
-  communicator* engine_comm = get_communicator(iengine, engine_comm_handle);
+  //MDI_Comm engine_comm_handle = library_get_matching_handle(comm);
+  //communicator* engine_comm = get_communicator(iengine, engine_comm_handle);
 
   // set the command
-  library_data* engine_lib = (library_data*) engine_comm->method_data;
-  snprintf(engine_lib->command, MDI_COMMAND_LENGTH_, "%s", command);
+  //library_data* engine_lib = (library_data*) engine_comm->method_data;
+  //snprintf(engine_lib->command, MDI_COMMAND_LENGTH_, "%s", command);
+  snprintf(libd->shared_state->command, MDI_COMMAND_LENGTH_, "%s", command);
 
   return 0;
 }
@@ -791,27 +816,36 @@ int library_execute_command(MDI_Comm comm) {
 
   // get the engine code to which this communicator connects
   library_data* libd = (library_data*) this->method_data;
-  int iengine = libd->connected_code;
-  code* engine_code = get_code(iengine);
+  //int iengine = libd->connected_code;
+  //code* engine_code = get_code(iengine);
 
-  MDI_Comm engine_comm_handle = library_get_matching_handle(comm);
-  communicator* engine_comm = get_communicator(iengine, engine_comm_handle);
-  library_data* engine_lib = (library_data*) engine_comm->method_data;
+  //MDI_Comm engine_comm_handle = library_get_matching_handle(comm);
+  MDI_Comm engine_comm_handle = libd->shared_state->engine_mdi_comm;
+  //communicator* engine_comm = get_communicator(iengine, engine_comm_handle);
+  //library_data* engine_lib = (library_data*) engine_comm->method_data;
 
   // set the current code to the engine
-  current_code = iengine;
+//  current_code = iengine;
+//  current_code = libd->shared_state->engine_code_id;
+  libd->shared_state->engine_activate_code( libd->shared_state->engine_code_id );
 
   // check if this command corresponds to one of MDI's standard built-in commands
-  int builtin_flag = general_builtin_command(engine_lib->command, engine_comm_handle);
+  //int builtin_flag = general_builtin_command(engine_lib->command, engine_comm_handle);
+  int builtin_flag = general_builtin_command(libd->shared_state->command, engine_comm_handle);
+  //int builtin_flag = 0;
 
   if ( builtin_flag == 0 ) {
     // call execute_command now
-    void* class_obj = engine_code->execute_command_obj;
-    ret = engine_code->execute_command(engine_lib->command,engine_comm_handle,class_obj);
+    //void* class_obj = engine_code->execute_command_obj;
+    void* class_obj = libd->shared_state->execute_command_obj;
+    //ret = engine_code->execute_command(engine_lib->command,engine_comm_handle,class_obj);
+    ret = libd->shared_state->execute_command(libd->shared_state->command, engine_comm_handle, class_obj);
   }
 
   // set the current code to the driver
-  current_code = idriver;
+//  current_code = idriver;
+//  current_code = libd->shared_state->driver_code_id;
+  libd->shared_state->driver_activate_code( libd->shared_state->driver_code_id );
 
   return ret;
 }
@@ -841,9 +875,10 @@ int library_send(const void* buf, int count, MDI_Datatype datatype, MDI_Comm com
   communicator* this = get_communicator(current_code, comm);
   library_data* libd = (library_data*) this->method_data;
 
-  code* other_code = get_code(libd->connected_code);
+  //code* other_code = get_code(libd->connected_code);
 
   // get the rank of this process on the engine
+  /*
   int engine_rank = 0;
   if ( this_code->is_library ) {
     engine_rank = this_code->intra_rank;
@@ -851,6 +886,8 @@ int library_send(const void* buf, int count, MDI_Datatype datatype, MDI_Comm com
   else {
     engine_rank = other_code->intra_rank;
   }
+  */
+  int engine_rank = libd->shared_state->intra_rank;
 
   // only send from rank 0
   if ( engine_rank == 0 ) {
@@ -869,9 +906,9 @@ int library_send(const void* buf, int count, MDI_Datatype datatype, MDI_Comm com
     if ( msg_flag == 1 ) { // message header
 
       // confirm that libd->buf is not already allocated
-      if ( libd->buf_allocated != 0 ) {
-	mdi_error("MDI recv buffer already allocated");
-	return 1;
+      if ( libd->shared_state->buf_allocated != 0 ) {
+        mdi_error("MDI recv buffer already allocated");
+        return 1;
       }
 
       // get the size of the message body, based on the header information
@@ -883,38 +920,38 @@ int library_send(const void* buf, int count, MDI_Datatype datatype, MDI_Comm com
       size_t body_stride;
       ret = datatype_info(body_type, &body_stride, &basetype);
       if ( ret != 0 ) {
-	mdi_error("datatype_info returned nonzero value in library_send");
-	return ret;
+        mdi_error("datatype_info returned nonzero value in library_send");
+        return ret;
       }
 
       int msg_bytes = ( (int)datasize * count ) + ( (int)body_stride * body_size );
 
       // allocate the memory required for the entire message
-      libd->buf = malloc( msg_bytes );
-      libd->buf_allocated = 1;
+      libd->shared_state->buf = malloc( msg_bytes );
+      libd->shared_state->buf_allocated = 1;
 
       // copy the header into libd->buf
-      memcpy(libd->buf, buf, nheader_actual * sizeof(int));
+      memcpy(libd->shared_state->buf, buf, nheader_actual * sizeof(int));
 
     }
     else if ( msg_flag == 2 ) { // message body
 
       // confirm that libd->buf has been allocated
       int has_header = 1;
-      if ( libd->buf_allocated == 0 ) {
-	// libd->buf has not been allocated, which means there is no header
-	has_header = 0;
-	libd->buf = malloc( datasize * count );
-	libd->buf_allocated = 1;
+      if ( libd->shared_state->buf_allocated == 0 ) {
+        // libd->buf has not been allocated, which means there is no header
+        has_header = 0;
+        libd->shared_state->buf = malloc( datasize * count );
+        libd->shared_state->buf_allocated = 1;
       }
 
       int offset = 0;
       if ( has_header == 1 ) {
-	offset = nheader_actual * sizeof(int);
+        offset = nheader_actual * sizeof(int);
       }
 
       // copy the body into libd->buf
-      memcpy((char*)libd->buf + offset, buf, datasize * count);
+      memcpy((char*)libd->shared_state->buf + offset, buf, datasize * count);
 
     }
     else {
@@ -929,7 +966,13 @@ int library_send(const void* buf, int count, MDI_Datatype datatype, MDI_Comm com
   // check whether the recipient code should now execute its command
   if ( msg_flag == 2 && libd->execute_on_send ) {
     // have the recipient code execute its command
-    library_execute_command(comm);
+    //library_execute_command(comm);
+
+    libd->shared_state->engine_activate_code( libd->shared_state->engine_code_id );
+
+    libd->shared_state->lib_execute_command(libd->shared_state->engine_mdi_comm);
+
+    libd->shared_state->driver_activate_code( libd->shared_state->driver_code_id );
 
     // turn off the execute_on_send flag
     libd->execute_on_send = 0;
@@ -963,12 +1006,13 @@ int library_recv(void* buf, int count, MDI_Datatype datatype, MDI_Comm comm, int
   communicator* this = get_communicator(current_code, comm);
   library_data* libd = (library_data*) this->method_data;
 
-  code* other_code = get_code(libd->connected_code);
-  MDI_Comm other_comm_handle = library_get_matching_handle(comm);
-  communicator* other_comm = get_communicator(libd->connected_code, other_comm_handle);
-  library_data* other_lib = (library_data*) other_comm->method_data;
+  //code* other_code = get_code(libd->connected_code);
+  //MDI_Comm other_comm_handle = library_get_matching_handle(comm);
+  //communicator* other_comm = get_communicator(libd->connected_code, other_comm_handle);
+  //library_data* other_lib = (library_data*) other_comm->method_data;
 
   // only recv from rank 0 of the engine
+  /*
   int engine_rank = 0;
   if ( this_code->is_library ) {
     engine_rank = this_code->intra_rank;
@@ -976,6 +1020,8 @@ int library_recv(void* buf, int count, MDI_Datatype datatype, MDI_Comm comm, int
   else {
     engine_rank = other_code->intra_rank;
   }
+  */
+  int engine_rank = libd->shared_state->intra_rank;
   if ( engine_rank != 0 ) {
     return 0;
   }
@@ -990,7 +1036,7 @@ int library_recv(void* buf, int count, MDI_Datatype datatype, MDI_Comm comm, int
   }
 
   // confirm that libd->buf is initialized
-  if ( other_lib->buf_allocated != 1 ) {
+  if ( libd->shared_state->buf_allocated != 1 ) {
     mdi_error("MDI send buffer is not allocated");
     return 1;
   }
@@ -1003,18 +1049,18 @@ int library_recv(void* buf, int count, MDI_Datatype datatype, MDI_Comm comm, int
 
     if ( msg_flag == 1 ) { // message header
 
-      memcpy(buf, other_lib->buf, count * datasize);
+      memcpy(buf, libd->shared_state->buf, count * datasize);
 
     }
     else if ( msg_flag == 2 ) { // message body
 
       int nheader = 4;
       int offset = nheader * sizeof(int);
-      memcpy(buf, (char*)other_lib->buf + offset, count * datasize);
+      memcpy(buf, (char*)libd->shared_state->buf + offset, count * datasize);
 
       // free the memory of libd->buf
-      free( other_lib->buf );
-      other_lib->buf_allocated = 0;
+      free( libd->shared_state->buf );
+      libd->shared_state->buf_allocated = 0;
 
     }
     else {
@@ -1040,11 +1086,21 @@ int communicator_delete_lib(void* comm) {
 
   // if this is the driver, delete the engine code
   if ( this_code->is_library == 0 ) {
-    delete_code(libd->connected_code);
-  }
+    //delete_code(libd->connected_code);
+    libd->shared_state->delete_engine( libd->shared_state->engine_code_id );
+
+    if ( libd->shared_state->plugin_argv_allocated ) {
+      free( libd->shared_state->plugin_argv );
+    }
+
+    // delete the method-specific information
+    if ( libd->shared_state->buf_allocated ) {
+      free( libd->shared_state->buf );
+    }
+
+    // delete the shared state, but only on the driver side
+    free( libd->shared_state );
 
-  if ( libd->plugin_argv_allocated ) {
-    free( libd->plugin_argv );
   }
 
   if ( libd->plugin_options_allocated ) {
@@ -1052,11 +1108,46 @@ int communicator_delete_lib(void* comm) {
     free( libd->plugin_unedited_options );
   }
 
-  // delete the method-specific information
-  if ( libd->buf_allocated ) {
-    free( libd->buf );
-  }
   free( libd );
 
   return 0;
 }
+
+
+/*! \brief Function to delete all of the engine's state
+ */
+int library_delete_engine(int code_id) {
+  delete_code(code_id);
+  if ( codes.size == 0 ) {
+    free(codes.data);
+  }
+  //free(methods.data);
+  return 0;
+}
+
+
+
+/*! \brief Function to set the plugin's state
+ */
+int library_set_state(void* state) {
+  shared_state_from_driver = (plugin_shared_state*) state;
+
+  // activate plugin mode
+  plugin_mode = 1;
+  //is_initialized = 1;
+
+  
+
+  MDI_Init(&shared_state_from_driver->plugin_argc, &shared_state_from_driver->plugin_argv);
+
+  return 0;
+}
+
+
+/*! \brief Function to set the active code
+ */
+int library_activate_code(int code_id) {
+  current_code = code_id;
+
+  return 0;
+}
